from __future__ import unicode_literals

from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool
from django.utils.translation import ugettext_lazy as _

from .menu import LeprikonMenu
from .urls import urlpatterns


@apphook_pool.register
class LeprikonApp(CMSApp):
    name = _('Leprikon')
    urls = [urlpatterns]
    app_name = 'leprikon'
    menus = [LeprikonMenu]



from __future__ import unicode_literals

from json import dumps

from django import forms
from django.db.models import Q
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _

from ..models.agegroup import AgeGroup
from ..models.courses import Course, CourseRegistration
from ..models.events import Event, EventRegistration
from ..models.fields import DAY_OF_WEEK
from ..models.place import Place
from ..models.roles import Leader, Parent, Participant
from ..models.subjects import Subject, SubjectGroup, SubjectType
from ..utils import get_age, get_birth_date
from .form import FormMixin
from .widgets import RadioSelectBootstrap


class SubjectFilterForm(FormMixin, forms.Form):
    q           = forms.CharField(label=_('Search term'), required=False)
    event_types = forms.ModelMultipleChoiceField(queryset=None, label=_('Event type'), required=False)
    groups      = forms.ModelMultipleChoiceField(queryset=None, label=_('Group'), required=False)
    leaders     = forms.ModelMultipleChoiceField(queryset=None, label=_('Leader'), required=False)
    places      = forms.ModelMultipleChoiceField(queryset=None, label=_('Place'), required=False)
    age_groups  = forms.ModelMultipleChoiceField(queryset=None, label=_('Age group'), required=False)
    days_of_week = forms.MultipleChoiceField(label=_('Day of week'),
                                             choices=tuple(sorted(DAY_OF_WEEK.items())), required=False)
    past        = forms.BooleanField(label=_('Include past subjects'), required=False)
    reg_active  = forms.BooleanField(label=_('Available for registration'), required=False)
    invisible   = forms.BooleanField(label=_('Show invisible'), required=False)

    _models = {
        SubjectType.COURSE: Course,
        SubjectType.EVENT:  Event,
    }

    def __init__(self, subject_type_type, subject_types, school_year, is_staff, data, **kwargs):
        super(SubjectFilterForm, self).__init__(data=data, **kwargs)
        self.subject_type_type = subject_type_type

        # pre filter subjects by initial params
        qs = self._models[subject_type_type].objects.filter(school_year=school_year)
        if len(subject_types) == 1:
            qs = qs.filter(subject_type=subject_types[0])
        else:
            qs = qs.filter(subject_type__in=subject_types)
        if not is_staff or 'invisible' not in data:
            qs = qs.filter(public=True)
        self.qs = qs

        subject_ids = set(qs.order_by().values_list('id', flat=True))
        if len(subject_types) == 1:
            del self.fields['event_types']
        else:
            self.fields['event_types'].queryset = SubjectType.objects.filter(id__in=[st.id for st in subject_types])
        self.fields['groups'].queryset  = SubjectGroup.objects.filter(subject_types__in=subject_types,
                                                                      subjects__id__in = subject_ids).distinct()
        self.fields['leaders'].queryset = (Leader.objects.filter(subjects__id__in=subject_ids).distinct()
                                           .order_by('user__first_name', 'user__last_name'))
        self.fields['places'].queryset  = Place.objects.filter(subjects__id__in=subject_ids).distinct()
        self.fields['age_groups'].queryset = AgeGroup.objects.filter(subjects__id__in=subject_ids).distinct()
        if subject_type_type != SubjectType.COURSE:
            del self.fields['days_of_week']
        if subject_type_type != SubjectType.EVENT:
            del self.fields['past']
        if not is_staff:
            del self.fields['invisible']

        for f in self.fields:
            self.fields[f].help_text = None

    def get_queryset(self):
        if not self.is_valid():
            return self.qs
        qs = self.qs
        for word in self.cleaned_data['q'].split():
            qs = qs.filter(
                Q(name__icontains = word) |
                Q(description__icontains = word)
            )
        if self.cleaned_data.get('event_types'):
            qs = qs.filter(subject_type__in = self.cleaned_data['event_types'])
        if self.cleaned_data['groups']:
            qs = qs.filter(groups__in = self.cleaned_data['groups'])
        if self.cleaned_data['places']:
            qs = qs.filter(place__in = self.cleaned_data['places'])
        if self.cleaned_data['leaders']:
            qs = qs.filter(leaders__in = self.cleaned_data['leaders'])
        if self.cleaned_data['age_groups']:
            qs = qs.filter(age_groups__in = self.cleaned_data['age_groups'])
        if self.subject_type_type == SubjectType.COURSE and self.cleaned_data['days_of_week']:
            qs = qs.filter(times__day_of_week__in = self.cleaned_data['days_of_week'])
        if self.subject_type_type == SubjectType.EVENT and not self.cleaned_data['past']:
            qs = qs.filter(end_date__gte = now())
        if self.cleaned_data['reg_active']:
            qs = qs.filter(reg_from__lte=now()).exclude(reg_to__lte=now()).exclude(price=None)
        return qs.distinct()



class SubjectForm(FormMixin, forms.ModelForm):

    class Meta:
        model = Subject
        fields = ['description', 'risks', 'plan', 'evaluation']



class RegistrationForm(FormMixin, forms.ModelForm):

    def __init__(self, subject, user, **kwargs):
        super(RegistrationForm, self).__init__(**kwargs)
        self.instance.subject = subject
        self.instance.user = user
        self.participants = user.leprikon_participants.exclude(
            birth_num__in=self.instance.subject.active_registrations.values_list('participant_birth_num', flat=True)
        )
        self.parents = user.leprikon_parents.all()

        self.fields['participant_age_group'].widget.choices.queryset = self.instance.subject.age_groups

        # dynamically required fields
        try:
            if get_age(get_birth_date(kwargs['data']['participant_birth_num'])) < 18:
                self.fields['has_parent1'].required = True
            else:
                self.fields['participant_phone'].required = True
                self.fields['participant_email'].required = True
        except:
            pass

        try:
            has_parent = ['has_parent1' in kwargs['data'], 'has_parent2' in kwargs['data']]
        except:
            has_parent = [False, False]
        for n in range(2):
            if has_parent[n]:
                for field in ['first_name', 'last_name', 'street', 'city', 'postal_code', 'phone', 'email']:
                    self.fields['parent{}_{}'.format(n + 1, field)].required = True

        # sub forms

        del kwargs['instance']

        class ParticipantSelectForm(FormMixin, forms.Form):
            participant = forms.ChoiceField(
                label   = _('Choose'),
                choices = [('new', _('new participant'))] + list((p.id, p) for p in self.participants),
                initial = 'new',
                widget  = RadioSelectBootstrap(),
            )
        kwargs['prefix'] = 'participant_select'
        self.participant_select_form = ParticipantSelectForm(**kwargs)

        class ParentSelectForm(FormMixin, forms.Form):
            parent = forms.ChoiceField(
                label   = _('Choose'),
                choices = [('new', _('new parent'))] + list((p.id, p) for p in self.parents),
                initial = 'new',
                widget  = RadioSelectBootstrap(),
            )
        kwargs['prefix'] = 'parent1_select'
        self.parent1_select_form = ParentSelectForm(**kwargs)
        kwargs['prefix'] = 'parent2_select'
        self.parent2_select_form = ParentSelectForm(**kwargs)

        kwargs['prefix'] = 'questions'
        self.questions_form = type(str('QuestionsForm'), (FormMixin, forms.Form), dict(
            (q.name, q.get_field()) for q in self.instance.subject.all_questions
        ))(**kwargs)

        kwargs['prefix'] = 'agreement'
        self.agreement_form = self.AgreementForm(**kwargs)

    def is_valid(self):
        return (
            self.participant_select_form.is_valid() and
            self.questions_form.is_valid() and
            self.parent1_select_form.is_valid() and
            self.parent2_select_form.is_valid() and
            self.agreement_form.is_valid() and
            super(RegistrationForm, self).is_valid()
        )

    def save(self, commit=True):
        self.instance.price = self.instance.subject.price
        self.instance.answers = dumps(self.questions_form.cleaned_data)
        super(RegistrationForm, self).save(commit)

        # send mail
        self.instance.send_mail()

        # save / update participant
        if self.participant_select_form.cleaned_data['participant'] == 'new':
            try:
                participant = Participant.objects.get(
                    user        = self.instance.user,
                    birth_num   = self.instance.participant.birth_num,
                )
            except Participant.DoesNotExist:
                participant = Participant()
                participant.user = self.instance.user
        else:
            participant = Participant.objects.get(id=self.participant_select_form.cleaned_data['participant'])
        for attr in ['first_name', 'last_name', 'birth_num', 'age_group', 'street', 'city', 'postal_code',
                     'citizenship', 'insurance', 'phone', 'email', 'school', 'school_other', 'school_class', 'health']:
            setattr(participant, attr, getattr(self.instance.participant, attr))
        participant.save()

        # save / update first parent
        if self.instance.parent1:
            if self.parent1_select_form.cleaned_data['parent'] == 'new':
                parent = Parent()
                parent.user = self.instance.user
            else:
                parent = Parent.objects.get(id=self.parent1_select_form.cleaned_data['parent'])
            for attr in ['first_name', 'last_name', 'street', 'city', 'postal_code', 'phone', 'email']:
                setattr(parent, attr, getattr(self.instance.parent1, attr))
            parent.save()

        # save / update second parent
        if self.instance.parent2:
            if self.parent2_select_form.cleaned_data['parent'] == 'new':
                parent = Parent()
                parent.user = self.instance.user
            else:
                parent = Parent.objects.get(id=self.parent2_select_form.cleaned_data['parent'])
            for attr in ['first_name', 'last_name', 'street', 'city', 'postal_code', 'phone', 'email']:
                setattr(parent, attr, getattr(self.instance.parent2, attr))
            parent.save()

    class AgreementForm(FormMixin, forms.Form):
        agreement = forms.BooleanField(
            label       = _('Terms and Conditions agreement'),
        )



class CourseRegistrationForm(RegistrationForm):

    class Meta:
        model = CourseRegistration
        exclude = ('subject', 'user', 'cancel_request', 'canceled')



class EventRegistrationForm(RegistrationForm):

    class Meta:
        model = EventRegistration
        exclude = ('subject', 'user', 'cancel_request', 'canceled')



class RegistrationAdminForm(forms.ModelForm):

    def __init__(self, data=None, *args, **kwargs):
        super(RegistrationAdminForm, self).__init__(data, *args, **kwargs)
        self.fields['participant_age_group'].widget.choices.queryset = kwargs['instance'].subject.age_groups

        try:
            age = get_age(get_birth_date(data['participant_birth_num']), kwargs['instance'].created.date())
        except:
            age = get_age(get_birth_date(kwargs['instance'].participant.birth_num), kwargs['instance'].created.date())
        if age < 18:
            self.fields['has_parent1'].required = True
        else:
            self.fields['participant_phone'].required = True
            self.fields['participant_email'].required = True

        try:
            has_parent = ['has_parent1' in data, 'has_parent2' in data]
        except:
            has_parent = [kwargs['instance'].has_parent1, kwargs['instance'].has_parent2]
        for n in range(2):
            if has_parent[n]:
                for field in ['first_name', 'last_name', 'street', 'city', 'postal_code', 'phone', 'email']:
                    self.fields['parent{}_{}'.format(n + 1, field)].required = True



from __future__ import unicode_literals

from collections import namedtuple

from django.utils.translation import ugettext_lazy as _

from ..conf import settings
from ..utils import currency


class PaymentStatus(namedtuple('_PaymentsStatus', ('price', 'discount', 'paid'))):

    @property
    def receivable(self):
        return self.price - self.discount

    @property
    def balance(self):
        return self.paid - self.receivable

    @property
    def color(self):
        if self.balance == 0:
            return settings.LEPRIKON_COLOR_PAID
        elif self.balance < 0:
            return settings.LEPRIKON_COLOR_NOTPAID
        else:
            return settings.LEPRIKON_COLOR_OVERPAID

    @property
    def title(self):
        if self.balance == 0:
            return _('paid')
        elif self.balance < 0:
            return _('{} let to pay').format(currency(-self.balance))
        else:
            return _('{} overpaid').format(currency(self.balance))

    def __repr__(self):
        return 'PaymentStatus(price={price}, discount={discount}, paid={paid}, balance={balance})'.format(
            price       = self.price,
            discount    = self.discount,
            paid        = self.paid,
            balance     = self.balance,
        )

    def __add__(self, other):
        if other == 0:
            return self
        return PaymentStatus(
            price       = self.price    + other.price,
            discount    = self.discount + other.discount,
            paid        = self.paid     + other.paid,
        )

    __radd__ = __add__



from __future__ import unicode_literals

from django.conf.urls import include, url
from django.db import transaction
from django.views.generic.base import RedirectView

from . import views
from .api import urls as api_urls
from .conf import settings

# dictionary of all LEPRIKON_URL_* attributes of settings
LEPRIKON_URL = dict(
    (attr.lower()[len('LEPRIKON_URL_'):], getattr(settings, attr))
    for attr in dir(settings) if attr.startswith('LEPRIKON_URL_')
)


def d_url(pattern, name):
    return url(pattern.format(**LEPRIKON_URL), transaction.atomic(getattr(views, name)), name=name)


urlpatterns = [
    url(r'^$', RedirectView.as_view(url='{summary}/'.format(**LEPRIKON_URL), permanent=True), name='index'),
    d_url(r'^{summary}/$',          'summary'),

    d_url(r'^{create_account}/$',           'user_create'),
    d_url(r'^{user}/$',                     'user_update'),
    d_url(r'^{user}/{password}/$',          'user_password'),
    d_url(r'^{login}/$',                    'user_login'),
    d_url(r'^{logout}/$',                   'user_logout'),
    d_url(r'^{password_reset}/$',           'password_reset'),
    d_url(r'^{password_reset}/done/$',      'password_reset_done'),
    d_url(r'^{password_reset}/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{{1,13}}-[0-9A-Za-z]{{1,20}})/$',
          'password_reset_confirm'),
    d_url(r'^{password_reset}/complete/$',  'password_reset_complete'),

    d_url(r'^{leader}/$',                                                           'leader_summary'),
    d_url(r'^{leader}/{journal}/(?P<pk>[0-9]+)/$',                                  'course_journal'),
    d_url(r'^{leader}/{journal}/{journalentry}/{add}/(?P<course>[0-9]+)/$',         'coursejournalentry_create'),
    d_url(r'^{leader}/{journal}/{journalentry}/(?P<pk>[0-9]+)/$',                   'coursejournalentry_update'),
    d_url(r'^{leader}/{journal}/{journalentry}/(?P<pk>[0-9]+)/{delete}/$',          'coursejournalentry_delete'),
    d_url(r'^{leader}/{journal}/{alternating}/$',                                   'course_alternating'),
    d_url(r'^{leader}/{timesheets}/$',                                              'timesheet_list'),
    d_url(r'^{leader}/{timesheets}/(?P<pk>[0-9]+)/$',                               'timesheet_detail'),
    d_url(r'^{leader}/{timesheets}/(?P<pk>[0-9]+)/{submit}/$',                      'timesheet_submit'),
    d_url(r'^{leader}/{timesheets}/{entry}/{add}/$',                                'timesheetentry_create'),
    d_url(r'^{leader}/{timesheets}/{entry}/(?P<pk>[0-9]+)/$',                       'timesheetentry_update'),
    d_url(r'^{leader}/{timesheets}/{entry}/(?P<pk>[0-9]+)/{delete}/$',              'timesheetentry_delete'),
    d_url(r'^{leader}/{timesheets}/{journalentry}/(?P<pk>[0-9]+)/$',                'coursejournalleaderentry_update'),
    d_url(r'^{leader}/{timesheets}/{journalentry}/(?P<pk>[0-9]+)/{delete}/$',       'coursejournalleaderentry_delete'),
    d_url(r'^{leader}/(?P<subject_type>[^/]+)/$',                                   'subject_list_mine'),
    d_url(r'^{leader}/(?P<subject_type>[^/]+)/(?P<pk>[0-9]+)/{edit}/$',             'subject_update'),
    d_url(r'^{leader}/(?P<subject_type>[^/]+)/(?P<pk>[0-9]+)/{registrations}/$',    'subject_registrations'),

    d_url(r'^{registrations}/$',                            'registration_list'),
    d_url(r'^{registrations}/(?P<slug>[^.]+).pdf$',         'registration_pdf'),
    d_url(r'^{registrations}/(?P<slug>[^/]+)/{cancel}/$',   'registration_cancel'),

    d_url(r'^{participants}/$',                             'participant_list'),
    d_url(r'^{participants}/{add}/$',                       'participant_create'),
    d_url(r'^{participants}/(?P<pk>[0-9]+)/$',              'participant_update'),
    d_url(r'^{participants}/{parent}/{add}/$',              'parent_create'),
    d_url(r'^{participants}/{parent}/(?P<pk>[0-9]+)/$',     'parent_update'),

    d_url(r'^{messages}/$',                                 'message_list'),
    d_url(r'^{messages}/(?P<slug>[^.]+)/$',                 'message_detail'),

    d_url(r'^{leaders}/$',                                  'leader_list'),
    # d_url(r'^{leaders}/(?P<slug>[^.]+)/$',                  'leader_detail'),

    d_url(r'^{school_year}/$',                              'school_year'),

    d_url(r'^{support}/$',                                  'support'),

    d_url(r'^{reports}/$',                                  'report_list'),
    d_url(r'^{reports}/{courses}/{payments}/$',             'report_course_payments'),
    d_url(r'^{reports}/{courses}/{payments_status}/$',      'report_course_payments_status'),
    d_url(r'^{reports}/{courses}/{stats}/$',                'report_course_stats'),
    d_url(r'^{reports}/{events}/{payments}/$',              'report_event_payments'),
    d_url(r'^{reports}/{events}/{payments_status}/$',       'report_event_payments_status'),
    d_url(r'^{reports}/{debtors}/$',                        'report_debtors'),

    d_url(r'^{terms_conditions}/$',                         'terms_conditions'),

    url(r'^api/', include(api_urls, 'api')),

    d_url(r'^(?P<subject_type>[^/]+)/$',                                 'subject_list'),
    d_url(r'^(?P<subject_type>[^/]+)/(?P<pk>[0-9]+)/$',                  'subject_detail'),
    d_url(r'^(?P<subject_type>[^/]+)/(?P<pk>[0-9]+)/{registration}/$',   'subject_registration_form'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-11-20 22:33
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0016_countries'),
    ]

    operations = [
        migrations.CreateModel(
            name='Insurance',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=10, unique=True, verbose_name='code')),
                ('name', models.CharField(max_length=150, verbose_name='name')),
            ],
            options={
                'ordering': ('code',),
                'verbose_name': 'insurance company',
                'verbose_name_plural': 'insurance companies',
            },
        ),
    ]
    for model_name in ['participant', 'clubregistration', 'eventregistration']:
        operations += [
            migrations.RenameField(
                model_name=model_name,
                old_name='insurance',
                new_name='insurance_old',
            ),
            migrations.AddField(
                model_name=model_name,
                name='insurance',
                field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='leprikon.Insurance', verbose_name='insurance'),
            ),
        ]



from __future__ import unicode_literals

from cms.plugin_pool import plugin_pool
from django.utils.translation import ugettext as _

from ..models.events import (
    EventListPlugin, EventPlugin, FilteredEventListPlugin,
)
from .base import LeprikonPluginBase


@plugin_pool.register_plugin
class LeprikonEventPlugin(LeprikonPluginBase):
    name    = _('Event')
    model   = EventPlugin
    raw_id_fields = ('event',)

    def get_render_template(self, context, instance, placeholder):
        return 'leprikon/cms/event/%s.html' % instance.template


@plugin_pool.register_plugin
class LeprikonEventListPlugin(LeprikonPluginBase):
    name    = _('Event list')
    model   = EventListPlugin
    filter_horizontal = ('age_groups', 'groups', 'leaders')

    def get_render_template(self, context, instance, placeholder):
        return 'leprikon/cms/event_list/%s.html' % instance.template


@plugin_pool.register_plugin
class LeprikonFilteredEventListPlugin(LeprikonPluginBase):
    name    = _('Event list with search form')
    model   = FilteredEventListPlugin
    render_template = 'leprikon/cms/event_list_filtered.html'



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-09-05 15:44
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('leprikon', '0010_club_discounts'),
    ]

    operations = [
        migrations.CreateModel(
            name='ClubRegistrationRequest',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='time of request')),
                ('contact', models.CharField(help_text='Enter phone number, e-mail address or other contact.', max_length=150, verbose_name='contact')),
                ('club', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='registration_requests', to='leprikon.Club', verbose_name='club')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='club_registration_requests', to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
            options={
                'ordering': ('created',),
                'verbose_name': 'club registration request',
                'verbose_name_plural': 'club registration requests',
            },
        ),
        migrations.CreateModel(
            name='EventRegistrationRequest',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='time of request')),
                ('contact', models.CharField(help_text='Enter phone number, e-mail address or other contact.', max_length=150, verbose_name='contact')),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='registration_requests', to='leprikon.Event', verbose_name='event')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='event_registration_requests', to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
            options={
                'ordering': ('created',),
                'verbose_name': 'event registration request',
                'verbose_name_plural': 'event registration requests',
            },
        ),
        migrations.AlterUniqueTogether(
            name='eventregistrationrequest',
            unique_together=set([('user', 'event')]),
        ),
        migrations.AlterUniqueTogether(
            name='clubregistrationrequest',
            unique_together=set([('user', 'club')]),
        ),
    ]



from __future__ import unicode_literals

from datetime import date

from django import forms
from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.utils.translation import (
    ugettext_lazy as _, ungettext_lazy as ungettext,
)

from ..models.courses import CourseJournalEntry, CourseJournalLeaderEntry
from ..models.roles import Leader
from ..models.timesheets import Timesheet, TimesheetPeriod
from ..utils import comma_separated
from .fields import ReadonlyField
from .form import FormMixin

User = get_user_model()


class CourseJournalLeaderEntryAdminForm(forms.ModelForm):

    class Meta:
        model = CourseJournalLeaderEntry
        fields = ['start', 'end']

    def __init__(self, *args, **kwargs):
        super(CourseJournalLeaderEntryAdminForm, self).__init__(*args, **kwargs)
        if self.instance.timesheet.submitted:
            try:
                del(self.fields['start'])
                del(self.fields['end'])
            except KeyError:
                pass

    def clean(self):
        cleaned_data = super(CourseJournalLeaderEntryAdminForm, self).clean()

        # readonly start end
        if self.instance.id and self.instance.timesheet.submitted:
            cleaned_data['start']   = self.instance.start
            cleaned_data['end']     = self.instance.end

        # check entry start and end
        errors = {}
        course_entry = self.instance.course_entry
        if cleaned_data['start'] < course_entry.start:
            errors['start'] = _('The course journal entry starts at {start}').format(
                start = course_entry.start,
            )
        if cleaned_data['end'] > course_entry.end:
            errors['end'] = _('The course journal entry ends at {end}').format(
                end = course_entry.end,
            )
        if errors:
            raise ValidationError(errors)

        return cleaned_data



class CourseJournalLeaderEntryForm(FormMixin, CourseJournalLeaderEntryAdminForm):

    def __init__(self, *args, **kwargs):
        super(CourseJournalLeaderEntryForm, self).__init__(*args, **kwargs)
        self.readonly_fields = [
            ReadonlyField(label=_('Course'),    value=self.instance.course_entry.course.name),
            ReadonlyField(label=_('Date'),      value=self.instance.course_entry.date),
            ReadonlyField(label=_('Leader'),    value=self.instance.timesheet.leader),
        ]
        if self.instance.timesheet.submitted:
            self.readonly_fields += [
                ReadonlyField(label=_('Start'), value=self.instance.start),
                ReadonlyField(label=_('End'),   value=self.instance.end),
            ]




class CourseJournalEntryAdminForm(forms.ModelForm):

    class Meta:
        model = CourseJournalEntry
        fields = ['date', 'start', 'end', 'agenda', 'registrations']

    def __init__(self, *args, **kwargs):
        self.course = kwargs.pop('course', None) or kwargs['instance'].course
        super(CourseJournalEntryAdminForm, self).__init__(*args, **kwargs)
        self.instance.course = self.course

        if self.instance.id:
            d = self.instance.date
            self.readonly_fields = [
                ReadonlyField(label=_('Date'),  value=d),
            ]
            try:
                del(self.fields['date'])
            except:
                pass
        else:
            last = self.instance.course.journal_entries.last()
            if last:
                last_end = last.datetime_end or last.date
            else:
                last_end = None
            next_time = self.instance.course.get_next_time(last_end)
            if next_time:
                self.initial['date']  = next_time.date
                self.initial['start'] = next_time.start
                self.initial['end']   = next_time.end
            else:
                self.initial['date']  = date.today()
            try:
                d = self.fields['date'].clean(kwargs['data']['date'])
            except:
                d = self.initial['date']

        self.fields['registrations'].widget.choices.queryset = self.instance.course.get_active_registrations(d)
        self.fields['registrations'].help_text = None

    def clean(self):
        start   = self.cleaned_data.get('start', None)
        end     = self.cleaned_data.get('end', None)

        # start and end must be both set or both None
        if start is None and end is not None:
            raise ValidationError({'start': _('If You fill in start time, You must fill end time too.')})
        elif end is None and start is not None:
            raise ValidationError({'end':   _('If You fill in end time, You must fill start time too.')})

        # check overlaping entries
        if start:
            qs = CourseJournalEntry.objects.filter(
                course      = self.instance.course,
                date        = self.cleaned_data.get('date', self.instance.date),
                start__lt   = end,
                end__gt     = start,
            )
            if self.instance.id:
                qs = qs.exclude(id=self.instance.id)
            if qs.exists():
                raise ValidationError(_('An overlaping entry has already been added in the course journal.'))

        # check submitted leader entries
        submitted_leader_entries = [
            e for e in self.instance.all_leader_entries
            if e.timesheet.submitted
        ]
        if submitted_leader_entries:
            max_start   = min(e.start for e in submitted_leader_entries)
            min_end     = max(e.end   for e in submitted_leader_entries)
            errors = {}
            if (start is None) or (start > max_start):
                errors['start'] = _('Some submitted timesheet entries start at {start}').format(
                    start = max_start,
                )
            if (end is None) or (end < min_end):
                errors['end'] = _('Some submitted timesheet entries end at {end}').format(
                    end = min_end,
                )
            if errors:
                raise ValidationError(errors)

        return self.cleaned_data



class CourseJournalEntryForm(FormMixin, CourseJournalEntryAdminForm):
    leaders     = forms.ModelMultipleChoiceField(Leader.objects, label=_('Leaders'), required=False)
    alternates  = forms.ModelMultipleChoiceField(Leader.objects, label=_('Alternates'), required=False)

    def __init__(self, *args, **kwargs):
        super(CourseJournalEntryForm, self).__init__(*args, **kwargs)
        self.readonly_fields = [
            ReadonlyField(label=_('Course'), value=self.course),
        ]

        # only allow to select leaders or alterates with not submitted timesheets
        leaders     = self.instance.course.all_leaders
        alternates  = [l for l in Leader.objects.all() if l not in leaders]

        self.fields['leaders'].widget.choices = tuple((l.id, l) for l in leaders)
        self.fields['leaders'].help_text = None
        self.fields['alternates'].widget.choices = tuple((l.id, l) for l in alternates)
        self.fields['alternates'].help_text = None

        if self.instance.id:
            self.initial['leaders']     = [l.id for l in self.instance.all_leaders]
            self.initial['alternates']  = [l.id for l in self.instance.all_alternates]
        else:
            self.initial['leaders']     = [l.id for l in leaders]

    def clean(self):
        self.cleaned_data = super(CourseJournalEntryForm, self).clean()
        self.cleaned_entries = []
        self.deleted_entries = []

        d   = self.cleaned_data.get('date', self.instance.date)
        if d is None:
            # no other validation makes sense without date
            return self.cleaned_data

        if 'start' in self.cleaned_data and 'end' in self.cleaned_data:
            errors = {}
            entries_by_leader = {
                'leaders': dict(
                    (entry.timesheet.leader, entry) for entry in self.instance.all_leader_entries
                    if entry.timesheet.leader in self.instance.all_leaders
                ),
                'alternates': dict(
                    (entry.timesheet.leader, entry) for entry in self.instance.all_leader_entries
                    if entry.timesheet.leader in self.instance.all_alternates
                ),
            }
            leaders_with_submitted_timesheets = {}
            period  = TimesheetPeriod.objects.for_date(d)

            for group in ('leaders', 'alternates'):
                leaders_with_submitted_timesheets[group] = []
                for leader in self.cleaned_data[group]:
                    if leader not in entries_by_leader[group]:
                        # try to create new leader entry
                        timesheet = Timesheet.objects.for_leader_and_date(leader=leader, date=d)
                        if timesheet.submitted:
                            # can not create entry
                            leaders_with_submitted_timesheets[group].append(leader)
                            continue
                        entry = CourseJournalLeaderEntry()
                        entry.course_entry = self.instance
                        entry.timesheet = Timesheet.objects.for_leader_and_date(leader=leader, date=d)
                        entry.start = self.cleaned_data['start']
                        entry.end   = self.cleaned_data['end']
                    else:
                        # try to update existing leader entry
                        entry = entries_by_leader[group].pop(leader)
                        if not entry.timesheet.submitted:
                            if self.cleaned_data['start'] != self.instance.start:
                                if entry.start == self.instance.start:
                                    entry.start = self.cleaned_data['start']
                                else:
                                    entry.start = max(entry.start, self.cleaned_data['start'])
                            if self.cleaned_data['end'] != self.instance.end:
                                if entry.end == self.instance.end:
                                    entry.end = self.cleaned_data['end']
                                else:
                                    entry.end = min(entry.end, self.cleaned_data['end'])
                    # store cleaned entry, or delete it, if it is broken by the update
                    if entry.start < entry.end:
                        self.cleaned_entries.append(entry)
                    elif entry.id:
                        self.deleted_entries.append(entry)
                # try to delete stale entries
                for entry in entries_by_leader[group].values():
                    if entry.timesheet.submitted:
                        # can not delete entry
                        leaders_with_submitted_timesheets[group].append(entry.timesheet.leader)
                        continue
                    # store deleted entry
                    self.deleted_entries.append(entry)

            if leaders_with_submitted_timesheets['leaders']:
                errors['leaders'] = ungettext(
                    'Leader {leaders} has already submitted timesheet for {period}.',
                    'Leaders {leaders} have already submitted timesheet for {period}.',
                    len(leaders_with_submitted_timesheets['leaders'])
                ).format(
                    leaders = comma_separated(leaders_with_submitted_timesheets['leaders']),
                    period  = period.name,
                )
            if leaders_with_submitted_timesheets['alternates']:
                errors['alternates'] = ungettext(
                    'Alternate {leaders} has already submitted timesheet for {period}.',
                    'Alternates {leaders} have already submitted timesheet for {period}.',
                    len(leaders_with_submitted_timesheets['alternates'])
                ).format(
                    leaders = comma_separated(leaders_with_submitted_timesheets['alternates']),
                    period  = period.name,
                )
            if errors:
                raise ValidationError(errors)

        return self.cleaned_data

    def save(self, commit=True):
        self.instance = super(CourseJournalEntryForm, self).save(commit)
        for entry in self.cleaned_entries:
            entry.course_entry = self.instance
            entry.save()
        for entry in self.deleted_entries:
            entry.delete()



from __future__ import unicode_literals

from cms.plugin_pool import plugin_pool
from django.utils.translation import ugettext as _

from ..models.courses import (
    CourseListPlugin, CoursePlugin, FilteredCourseListPlugin,
)
from .base import LeprikonPluginBase


@plugin_pool.register_plugin
class LeprikonCoursePlugin(LeprikonPluginBase):
    name    = _('Course')
    model   = CoursePlugin
    raw_id_fields = ('course',)

    def get_render_template(self, context, instance, placeholder):
        return 'leprikon/cms/course/%s.html' % instance.template


@plugin_pool.register_plugin
class LeprikonCourseListPlugin(LeprikonPluginBase):
    name    = _('Course list')
    model   = CourseListPlugin
    filter_horizontal = ('age_groups', 'groups', 'leaders')

    def get_render_template(self, context, instance, placeholder):
        return 'leprikon/cms/course_list/%s.html' % instance.template


@plugin_pool.register_plugin
class LeprikonFilteredCourseListPlugin(LeprikonPluginBase):
    name    = _('Course list with search form')
    model   = FilteredCourseListPlugin
    render_template = 'leprikon/cms/course_list_filtered.html'



from __future__ import unicode_literals

from django.core.mail import mail_managers
from django.utils.translation import ugettext_lazy as _

from ..forms.support import SupportForm
from .generic import FormView


class SupportView(FormView):
    form_class  = SupportForm
    title       = _('Support')
    message     = _('Your question has been sent to our support team. Thank You.')

    def form_valid(self, form):
        question = form.cleaned_data['question']
        if question.strip():
            mail_managers(
                _('Question from web'),
                _('User {user} asks:\n{question}').format(
                    user     = self.request.user.get_username(),
                    question = question,
                ),
            )
        return super(SupportView, self).form_valid(form)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2017-01-12 22:17
from __future__ import unicode_literals

from django.db import migrations
import leprikon.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0023_registration_price'),
    ]

    operations = [
        migrations.AlterField(
            model_name='club',
            name='price',
            field=leprikon.models.fields.PriceField(blank=True, decimal_places=0, max_digits=10, null=True, verbose_name='price'),
        ),
        migrations.AlterField(
            model_name='event',
            name='price',
            field=leprikon.models.fields.PriceField(blank=True, decimal_places=0, max_digits=10, null=True, verbose_name='price'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-11-20 15:15
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0014_gender'),
    ]

    operations = [
        migrations.AlterField(
            model_name='leader',
            name='user',
            field=models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, related_name='leprikon_leader', to=settings.AUTH_USER_MODEL, verbose_name='user'),
        ),
    ]



from __future__ import unicode_literals

from collections import namedtuple

from django.core.urlresolvers import reverse_lazy as reverse
from django.db.models import Sum
from django.template.response import TemplateResponse
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from . import ReportBaseView
from ...conf import settings
from ...forms.reports.courses import (
    CoursePaymentsForm, CoursePaymentsStatusForm, CourseStatsForm,
)
from ...models.agegroup import AgeGroup
from ...models.courses import Course, CourseRegistration
from ...models.roles import Participant
from ...models.subjects import SubjectPayment, SubjectType


class ReportCoursePaymentsView(ReportBaseView):
    form_class      = CoursePaymentsForm
    template_name   = 'leprikon/reports/course_payments.html'
    title           = _('Course payments')
    submit_label    = _('Show')
    back_url        = reverse('leprikon:report_list')

    def form_valid(self, form):
        context = form.cleaned_data
        context['form'] = form
        context['payments'] = SubjectPayment.objects.filter(
            registration__subject__subject_type__subject_type=SubjectType.COURSE,
            created__gte=context['date_start'],
            created__lte=context['date_end'],
        )
        context['sum'] = context['payments'].aggregate(Sum('amount'))['amount__sum']
        return TemplateResponse(self.request, self.template_name, self.get_context_data(**context))



class ReportCoursePaymentsStatusView(ReportBaseView):
    form_class      = CoursePaymentsStatusForm
    template_name   = 'leprikon/reports/course_payments_status.html'
    title           = _('Course payments status')
    submit_label    = _('Show')
    back_url        = reverse('leprikon:report_list')

    CoursePaymentsStatusSums = namedtuple('CoursePaymentsStatusSums', ('registrations', 'status'))

    def form_valid(self, form):
        context = form.cleaned_data
        context['form'] = form
        context['reports'] = [
            self.Report(course, context['date'])
            for course in Course.objects.filter(school_year=self.request.school_year)
        ]
        context['sum'] = self.CoursePaymentsStatusSums(
            registrations   = sum(len(r.registrations)  for r in context['reports']),
            status          = sum(r.status              for r in context['reports']),
        )
        return TemplateResponse(self.request, self.template_name, self.get_context_data(**context))

    class Report:
        def __init__(self, course, d):
            self.course = course
            self.date = d

        @cached_property
        def periods(self):
            return list(self.course.periods.filter(start__lte=self.date))

        @cached_property
        def registrations(self):
            return list(self.course.registrations.filter(
                created__lte=self.date,
            ))

        RegPaymentStatuses = namedtuple('RegPaymentStatuses', ('registration', 'status'))

        @cached_property
        def registration_statuses(self):
            return [
                self.RegPaymentStatuses(
                    registration = registration,
                    status       = registration.courseregistration.get_payment_statuses(self.date).partial,
                )
                for registration in self.registrations
            ]

        @cached_property
        def status(self):
            return sum(rs.status for rs in self.registration_statuses)



class ReportCourseStatsView(ReportBaseView):
    form_class      = CourseStatsForm
    template_name   = 'leprikon/reports/course_stats.html'
    title           = _('Course statistics')
    submit_label    = _('Show')
    back_url        = reverse('leprikon:report_list')

    ReportItem      = namedtuple('ReportItem', ('age_group', 'all', 'boys', 'girls', 'local', 'eu', 'noneu'))

    EU_countries    = [
        'AT', 'BE', 'BG', 'CY', 'CZ', 'DK', 'EE', 'FI', 'FR', 'DE', 'GR', 'HU', 'IE', 'IT',
        'LV', 'LT', 'LU', 'MT', 'NL', 'PL', 'PT', 'RO', 'SK', 'SI', 'ES', 'SE', 'GB',
    ]
    EU_countries.remove(settings.LEPRIKON_COUNTRY)

    def form_valid(self, form):
        d       = form.cleaned_data['date']
        context = form.cleaned_data
        context['form'] = form

        courses = Course.objects.filter(periods__start__lte=d, periods__end__gte=d).distinct()
        context['courses_count'] = courses.count()

        registrations = CourseRegistration.objects.filter(subject__in=courses, created__lte=d).exclude(canceled__lte=d)

        context['registrations_counts'] = self.ReportItem(
            age_group=None,
            all=registrations.count(),
            boys=registrations.filter(participant_gender=Participant.MALE).count(),
            girls=registrations.filter(participant_gender=Participant.FEMALE).count(),
            local=registrations.filter(participant_citizenship=settings.LEPRIKON_COUNTRY).count(),
            eu=registrations.filter(participant_citizenship__in=self.EU_countries).count(),
            noneu=registrations.exclude(participant_citizenship__in=self.EU_countries +
                                        [settings.LEPRIKON_COUNTRY]).count(),
        )
        context['registrations_counts_by_age_groups'] = []
        for age_group in AgeGroup.objects.all():
            regs = registrations.filter(participant_age_group=age_group)
            context['registrations_counts_by_age_groups'].append(self.ReportItem(
                age_group=age_group,
                all=regs.count(),
                boys=regs.filter(participant_gender=Participant.MALE).count(),
                girls=regs.filter(participant_gender=Participant.FEMALE).count(),
                local=regs.filter(participant_citizenship=settings.LEPRIKON_COUNTRY).count(),
                eu=regs.filter(participant_citizenship__in=self.EU_countries).count(),
                noneu=regs.exclude(participant_citizenship__in=self.EU_countries +
                                   [settings.LEPRIKON_COUNTRY]).count(),
            ))

        return TemplateResponse(self.request, self.template_name, self.get_context_data(**context))



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2017-01-13 17:32
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import filer.fields.file


def populate_club_types(apps, schema_editor):
    Club = apps.get_model('leprikon', 'club')
    ClubListPlugin = apps.get_model('leprikon', 'clublistplugin')
    FilteredClubListPlugin = apps.get_model('leprikon', 'filteredclublistplugin')
    ClubType = apps.get_model('leprikon', 'clubtype')

    club_type = ClubType.objects.create(name='Kroužky', slug='krouzky')

    Club.objects.update(club_type=club_type)
    ClubListPlugin.objects.update(club_type=club_type)
    for fclp in FilteredClubListPlugin.objects.all():
        fclp.club_types.add(club_type)


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0001_initial'),
        ('leprikon', '0024_price_not_mandatory'),
    ]

    operations = [
        migrations.CreateModel(
            name='ClubType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150, verbose_name='name')),
                ('slug', models.SlugField()),
                ('order', models.IntegerField(blank=True, default=0, verbose_name='order')),
                ('questions', models.ManyToManyField(blank=True, help_text='Add additional questions to be asked in the registration form.', to='leprikon.Question', verbose_name='additional questions')),
            ],
            options={
                'ordering': ('order',),
                'verbose_name': 'club type',
                'verbose_name_plural': 'club types',
            },
        ),
        migrations.CreateModel(
            name='ClubTypeAttachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.IntegerField(blank=True, default=0, verbose_name='order')),
                ('club_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attachments', to='leprikon.ClubType', verbose_name='club type')),
                ('file', filer.fields.file.FilerFileField(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='filer.File')),
            ],
            options={
                'ordering': ('order',),
                'verbose_name': 'attachment',
                'verbose_name_plural': 'attachments',
            },
        ),
        migrations.RenameField(
            model_name='eventtypeattachment',
            old_name='event',
            new_name='event_type',
        ),
        migrations.AddField(
            model_name='club',
            name='club_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='clubs', to='leprikon.ClubType', verbose_name='club type'),
        ),
        migrations.AddField(
            model_name='clublistplugin',
            name='club_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='leprikon.ClubType', verbose_name='club type'),
        ),
        migrations.AddField(
            model_name='filteredclublistplugin',
            name='club_types',
            field=models.ManyToManyField(to='leprikon.ClubType', verbose_name='club type'),
        ),
        migrations.RunPython(populate_club_types),
        migrations.AlterField(
            model_name='club',
            name='club_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='clubs', to='leprikon.ClubType', verbose_name='club type'),
        ),
        migrations.AlterField(
            model_name='clublistplugin',
            name='club_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='leprikon.ClubType', verbose_name='club type'),
        ),
    ]



from __future__ import unicode_literals

from django.forms.widgets import (
    CheckboxFieldRenderer, CheckboxSelectMultiple, RadioFieldRenderer,
    RadioSelect,
)
from django.utils.safestring import mark_safe


class BootstrapRenderer:

    def render_option(self, value, label, i):
        widget = self.choice_input_class(self.name, self.value, self.attrs.copy(), (value, label), i)
        row    = (
            '<div class="row select">'
            '    <div class="col-md-1 right">{tag}</div>'
            '    <div class="col-md-11">'
            '        <label class="form-control" style="font-weight:normal" for="{id}">{label}</label>'
            '    </div>'
            '</div>'
        )
        return row.format(
            tag     = widget.tag(),
            id      = widget.attrs['id'],
            label   = widget.choice_label,
        )

    def render(self):
        rows    = []
        i       = 0
        for value, label in self.choices:
            if isinstance(label, (list, tuple)):
                for v, l in label:
                    rows.append(self.render_option(v, l, i))
                    i += 1
            else:
                rows.append(self.render_option(value, label, i))
            i += 1
        return mark_safe('\n'.join(rows))



class RadioFieldRendererBootstrap(BootstrapRenderer, RadioFieldRenderer):
    ''' Renders RadioSelect in a nice table '''



class CheckboxFieldRendererBootstrap(BootstrapRenderer, CheckboxFieldRenderer):
    ''' Renders CheckboxSelectMultiple in a nice table '''



class RadioSelectBootstrap(RadioSelect):
    renderer = RadioFieldRendererBootstrap



class CheckboxSelectMultipleBootstrap(CheckboxSelectMultiple):
    renderer = CheckboxFieldRendererBootstrap



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-08-11 08:40
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0008_cms_plugins'),
    ]

    operations = [
        migrations.AlterField(
            model_name='clublistplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='leprikon_clublistplugin', serialize=False, to='cms.CMSPlugin'),
        ),
        migrations.AlterField(
            model_name='clubplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='leprikon_clubplugin', serialize=False, to='cms.CMSPlugin'),
        ),
        migrations.AlterField(
            model_name='eventlistplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='leprikon_eventlistplugin', serialize=False, to='cms.CMSPlugin'),
        ),
        migrations.AlterField(
            model_name='eventplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='leprikon_eventplugin', serialize=False, to='cms.CMSPlugin'),
        ),
        migrations.AlterField(
            model_name='filteredclublistplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='leprikon_filteredclublistplugin', serialize=False, to='cms.CMSPlugin'),
        ),
        migrations.AlterField(
            model_name='filteredeventlistplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='leprikon_filteredeventlistplugin', serialize=False, to='cms.CMSPlugin'),
        ),
        migrations.AlterField(
            model_name='filteredleaderlistplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='leprikon_filteredleaderlistplugin', serialize=False, to='cms.CMSPlugin'),
        ),
        migrations.AlterField(
            model_name='leaderlistplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='leprikon_leaderlistplugin', serialize=False, to='cms.CMSPlugin'),
        ),
        migrations.AlterField(
            model_name='leaderplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='leprikon_leaderplugin', serialize=False, to='cms.CMSPlugin'),
        ),
    ]



from __future__ import unicode_literals


class FormMixin(object):
    required_css_class = 'required'

    def __init__(self, *args, **kwargs):
        super(FormMixin, self).__init__(*args, **kwargs)
        for f in self.fields:
            # add form-control to classes
            classes = set(self.fields[f].widget.attrs.get('class', '').split())
            classes.add('form-control')
            self.fields[f].widget.attrs['class'] = ' '.join(classes)
            # add data-type
            if 'data-type' not in self.fields[f].widget.attrs:
                self.fields[f].widget.attrs['data-type'] = self.fields[f].__class__.__name__



from __future__ import unicode_literals

from .models.roles import Leader
from .models.schoolyear import SchoolYear


class school_year(object):
    def __get__(self, request, type=None):
        if request is None:
            return self
        try:
            return request._leprikon_school_year
        except AttributeError:
            years = SchoolYear.objects
            if not request.user.is_staff:
                years = years.filter(active=True)
            try:
                # return year stored in the session
                request._leprikon_school_year = years.get(id=request.session['school_year_id'])
            except (KeyError, SchoolYear.DoesNotExist):
                request._leprikon_school_year = SchoolYear.objects.get_current()
        return request._leprikon_school_year

    def __set__(self, request, school_year):
        if request:
            request._leprikon_school_year = school_year
            request.session['school_year_id'] = school_year.id

    def __delete__(self, request):
        if request:
            del(request._leprikon_school_year)



class leader(object):
    def __get__(self, request, type=None):
        if request is None:
            return self
        try:
            return request._leprikon_leader
        except AttributeError:
            try:
                request._leprikon_leader = request.user.leprikon_leader
            except (AttributeError, Leader.DoesNotExist):
                request._leprikon_leader = None
        return request._leprikon_leader

    def __set__(self, request, leader):
        pass

    def __delete__(self, request):
        if request:
            del(request._leprikon_leader)



class LeprikonMiddleware(object):

    def __init__(self, get_response=None):
        self.get_response = get_response

    def __call__(self, request):
        type(request).school_year = school_year()
        type(request).leader = leader()
        return self.get_response(request)

    # django<1.10 method
    def process_request(self, request):
        type(request).school_year = school_year()
        type(request).leader = leader()



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-05 13:18
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0001_initial'),
        ('leprikon', '0007_messages'),
    ]

    operations = [
        migrations.CreateModel(
            name='ClubPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('template', models.CharField(choices=[('default', 'Default')], default='default', help_text='The template used to render plugin.', max_length=100, verbose_name='template')),
                ('club', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='leprikon.Club', verbose_name='club')),
            ],
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='EventPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('template', models.CharField(choices=[('default', 'Default')], default='default', help_text='The template used to render plugin.', max_length=100, verbose_name='template')),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='leprikon.Event', verbose_name='event')),
            ],
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='FilteredLeaderListPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('school_year', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='leprikon.SchoolYear', verbose_name='school year')),
            ],
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='LeaderListPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('template', models.CharField(choices=[('default', 'Default')], default='default', help_text='The template used to render plugin.', max_length=100, verbose_name='template')),
                ('club', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='leprikon.Club', verbose_name='club')),
                ('event', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='leprikon.Event', verbose_name='event')),
                ('school_year', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='leprikon.SchoolYear', verbose_name='school year')),
            ],
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='LeaderPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('template', models.CharField(choices=[('default', 'Default')], default='default', help_text='The template used to render plugin.', max_length=100, verbose_name='template')),
                ('leader', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='leprikon.Leader', verbose_name='leader')),
            ],
            bases=('cms.cmsplugin',),
        ),
        migrations.RenameModel(
            old_name='LeprikonClubListPlugin',
            new_name='ClubListPlugin',
        ),
        migrations.RenameModel(
            old_name='LeprikonEventListPlugin',
            new_name='EventListPlugin',
        ),
        migrations.RenameModel(
            old_name='LeprikonFilteredClubListPlugin',
            new_name='FilteredClubListPlugin',
        ),
        migrations.RenameModel(
            old_name='LeprikonFilteredEventListPlugin',
            new_name='FilteredEventListPlugin',
        ),
    ]



from __future__ import unicode_literals

from collections import namedtuple
from datetime import date, datetime, timedelta

from cms.models import CMSPlugin
from django.core.urlresolvers import reverse_lazy as reverse
from django.db import models
from django.dispatch import receiver
from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils.formats import date_format
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from djangocms_text_ckeditor.fields import HTMLField

from ..conf import settings
from ..utils import comma_separated
from .agegroup import AgeGroup
from .fields import DAY_OF_WEEK, DayOfWeekField
from .roles import Leader
from .schoolyear import SchoolYear, SchoolYearPeriod
from .startend import StartEndMixin
from .subjects import (
    Subject, SubjectDiscount, SubjectGroup, SubjectRegistration, SubjectType,
)
from .utils import PaymentStatus


class Course(Subject):
    unit    = models.CharField(_('unit'), max_length=150)
    periods = models.ManyToManyField(SchoolYearPeriod, verbose_name=_('periods'), related_name='courses')

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('name',)
        verbose_name        = _('course')
        verbose_name_plural = _('courses')

    @cached_property
    def all_times(self):
        return list(self.times.all())

    @cached_property
    def all_periods(self):
        return list(self.periods.all())

    @cached_property
    def all_journal_periods(self):
        return list(CourseJournalPeriod(self, period) for period in self.all_periods)

    @cached_property
    def all_journal_entries(self):
        return list(self.journal_entries.all())

    def get_current_period(self):
        return self.periods.filter(end__gte=date.today()).first() or self.periods.last()

    def get_times_list(self):
        return comma_separated(self.all_times)
    get_times_list.short_description = _('times')

    def get_next_time(self, now = None):
        try:
            return min(t.get_next_time(now) for t in self.all_times)
        except ValueError:
            return None

    @property
    def registrations_history_registrations(self):
        return CourseRegistration.objects.filter(course_history__course=self).distinct()

    @property
    def inactive_registrations(self):
        return (CourseRegistration.objects.filter(course_history__course=self)
                .exclude(id__in=self.active_registrations.all()).distinct())

    def get_active_registrations(self, d):
        ids = (CourseRegistrationHistory.objects.filter(course=self, start__lte=d)
               .exclude(end__lt=d).values_list('registration_id', flat=True))
        return CourseRegistration.objects.filter(id__in=ids).exclude(canceled__lt=d).distinct()

    def copy_to_school_year(old, school_year):
        new = Course.objects.get(id=old.id)
        new.id = None
        new.school_year = school_year
        new.public      = False
        new.evaluation  = ''
        new.note        = ''
        new.save()
        new.groups      = old.groups.all()
        new.age_groups  = old.age_groups.all()
        new.leaders     = old.leaders.all()
        new.questions   = old.questions.all()
        new.times       = old.times.all()
        new.attachments = old.attachments.all()
        year_offset = school_year.year - old.school_year.year
        if new.reg_from:
            try:
                new.reg_from = datetime(
                    new.reg_from.year + year_offset,
                    new.reg_from.month,
                    new.reg_from.day,
                    new.reg_from.hour,
                    new.reg_from.minute,
                    new.reg_from.second,
                )
            except ValueError:
                # handle leap-year
                new.reg_from = datetime(
                    new.reg_from.year + year_offset,
                    new.reg_from.month,
                    new.reg_from.day - 1,
                    new.reg_from.hour,
                    new.reg_from.minute,
                    new.reg_from.second,
                )
        if new.reg_to:
            try:
                new.reg_to = datetime(
                    new.reg_to.year + year_offset,
                    new.reg_to.month,
                    new.reg_to.day,
                    new.reg_to.hour,
                    new.reg_to.minute,
                    new.reg_to.second,
                )
            except ValueError:
                # handle leap-year
                new.reg_to = datetime(
                    new.reg_to.year + year_offset,
                    new.reg_to.month,
                    new.reg_to.day - 1,
                    new.reg_to.hour,
                    new.reg_to.minute,
                    new.reg_to.second,
                )
        for period in old.all_periods:
            try:
                start = date(period.start.year + year_offset, period.start.month, period.start.day)
            except ValueError:
                # handle leap-year
                start = date(period.start.year + year_offset, period.start.month, period.start.day - 1)
            try:
                end   = date(period.end.year   + year_offset, period.end.month,   period.end.day)
            except ValueError:
                # handle leap-year
                end   = date(period.end.year   + year_offset, period.end.month,   period.end.day - 1)
            new.periods.add(SchoolYearPeriod.get_or_create(
                school_year=school_year, name=period.name, start=start, end=end
            )[0])
        return new



class CourseJournalPeriod:

    def __init__(self, course, period):
        self.course = course
        self.period = period

    @property
    def journal_entries(self):
        return self.course.journal_entries.filter(date__gte=self.period.start, date__lte=self.period.end)

    @cached_property
    def all_journal_entries(self):
        return list(self.journal_entries.all())

    @cached_property
    def all_registrations(self):
        return list(self.course.registrations_history_registrations.filter(created__lt=self.period.end))

    @cached_property
    def all_alternates(self):
        alternates = set()
        for entry in self.all_journal_entries:
            for alternate in entry.all_alternates:
                alternates.add(alternate)
        return list(alternates)

    PresenceRecord = namedtuple('PresenceRecord', ('name', 'presences'))

    def get_participant_presences(self):
        return [
            self.PresenceRecord(
                reg.participant,
                [
                    reg in entry.all_registrations
                    for entry in self.all_journal_entries
                ]
            ) for reg in self.all_registrations
        ]

    def get_leader_presences(self):
        return [
            self.PresenceRecord(
                leader,
                [
                    entry.all_leader_entries_by_leader.get(leader, None)
                    for entry in self.all_journal_entries
                ]
            ) for leader in self.course.all_leaders
        ]

    def get_alternate_presences(self):
        return [
            self.PresenceRecord(
                alternate,
                [
                    entry.all_leader_entries_by_leader.get(alternate, None)
                    for entry in self.all_journal_entries
                ]
            ) for alternate in self.all_alternates
        ]




@python_2_unicode_compatible
class CourseTime(StartEndMixin, models.Model):
    course      = models.ForeignKey(Course, verbose_name=_('course'), related_name='times')
    day_of_week = DayOfWeekField(_('day of week'))
    start       = models.TimeField(_('start time'), blank=True, null=True)
    end         = models.TimeField(_('end time'), blank=True, null=True)

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('day_of_week', 'start')
        verbose_name        = _('time')
        verbose_name_plural = _('times')

    def __str__(self):
        if self.start is not None and self.end is not None:
            return _('{day}, {start:%H:%M} - {end:%H:%M}').format(
                day     = self.day,
                start   = self.start,
                end     = self.end,
            )
        elif self.start is not None:
            return _('{day}, {start:%H:%M}').format(
                day     = self.day,
                start   = self.start,
            )
        else:
            return force_text(self.day)

    @cached_property
    def day(self):
        return DAY_OF_WEEK[self.day_of_week]

    Time = namedtuple('Time', ('date', 'start', 'end'))

    def get_next_time(self, now = None):
        now = now or datetime.now()
        daydelta = (self.day_of_week - now.isoweekday()) % 7
        if daydelta == 0 and (isinstance(now, date) or self.start is None or self.start <= now.time()):
            daydelta = 7
        if isinstance(now, datetime):
            next_date = now.date() + timedelta(daydelta)
        else:
            next_date = now + timedelta(daydelta)
        return self.Time(
            date    = next_date,
            start   = self.start,
            end     = self.end,
        )



class CourseRegistration(SubjectRegistration):

    class Meta:
        app_label           = 'leprikon'
        verbose_name        = _('course registration')
        verbose_name_plural = _('course registrations')

    @property
    def periods(self):
        if self.canceled:
            return self.subject.course.periods.filter(end__gt=self.created, start__lt=self.canceled)
        else:
            return self.subject.course.periods.filter(end__gt=self.created)

    @cached_property
    def all_periods(self):
        return list(self.periods.all())

    @cached_property
    def all_discounts(self):
        return list(self.discounts.all())

    PeriodPaymentStatus = namedtuple('PeriodPaymentStatus', ('period', 'status'))

    def get_period_payment_statuses(self, d=None):
        paid = self.get_paid(d)
        for counter, period in enumerate(self.all_periods, start=1):
            discount = sum(
                discount.amount
                for discount in filter(
                    lambda discount: discount.period == period and (d is None or discount.created <= d),
                    self.all_discounts
                )
            )
            yield self.PeriodPaymentStatus(
                period  = period,
                status  = PaymentStatus(
                    price       = self.price,
                    discount    = discount,
                    paid        = min(self.price - discount, paid) if counter < len(self.all_periods) else paid,
                ),
            )
            paid = max(paid - (self.price - discount), 0)

    @cached_property
    def payment_statuses(self):
        return self.get_payment_statuses()

    PaymentStatuses = namedtuple('PaymentStatuses', ('partial', 'total'))

    def get_payment_statuses(self, d=None):
        if d is None:
            d = date.today()
        if self.approved:
            if self.approved.date() <= d:
                partial_price = self.price * len(list(filter(lambda p: p.start <= d, self.all_periods)))
            else:
                partial_price = 0
            total_price = self.price * len(self.all_periods)
        else:
            partial_price = 0
            total_price = 0
        partial_discount    = sum(
            discount.amount
            for discount in filter(lambda discount: discount.period.start <= d, self.all_discounts)
        )
        total_discount  = sum(discount.amount for discount in self.all_discounts)
        paid            = self.get_paid(d)
        return self.PaymentStatuses(
            partial = PaymentStatus(price=partial_price, discount=partial_discount, paid=paid),
            total   = PaymentStatus(price=total_price, discount=total_discount, paid=paid),
        )



class CourseDiscount(SubjectDiscount):
    registration    = models.ForeignKey(CourseRegistration, verbose_name=_('registration'),
                                        related_name='discounts', on_delete=models.PROTECT)
    period          = models.ForeignKey(SchoolYearPeriod, verbose_name=_('period'),
                                        related_name='discounts', on_delete=models.PROTECT)



@python_2_unicode_compatible
class CourseRegistrationHistory(StartEndMixin, models.Model):
    registration = models.ForeignKey(CourseRegistration, verbose_name=_('course'), editable=False,
                                     related_name='course_history', on_delete=models.PROTECT)
    course = models.ForeignKey(Course, verbose_name=_('course'), editable=False,
                               related_name='registrations_history', on_delete=models.PROTECT)
    start = models.DateField()
    end = models.DateField(blank=True, null=True)

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('start',)
        verbose_name        = _('course registration history')
        verbose_name_plural = _('course registration history')

    def __str__(self):
        return '{course}, {start} - {end}'.format(
            course  = self.course.name,
            start   = date_format(self.start, 'SHORT_DATE_FORMAT'),
            end     = date_format(self.end, 'SHORT_DATE_FORMAT') if self.end else _('now'),
        )

    @property
    def course_journal_entries(self):
        qs = CourseJournalEntry.objects.filter(course=self.course, date__gte=self.start)
        if self.end:
            return qs.filter(date__lte=self.end)
        else:
            return qs

    def save(self, *args, **kwargs):
        if self.id:
            original = self.__class__.objects.get(id=self.id)
            min_journal_date = original.course_journal_entries.aggregate(models.Min('date'))['date__min']
            max_journal_date = original.course_journal_entries.aggregate(models.Max('date'))['date__max']
            # if journal entry exists, start must be set and lower or equal to min journal date
            if min_journal_date and self.start > min_journal_date:
                self.start = min_journal_date
            # end can not be lower than max journal date
            if self.end and max_journal_date and self.end < max_journal_date:
                self.end = max_journal_date
        super(CourseRegistrationHistory, self).save(*args, **kwargs)



@receiver(models.signals.post_save, sender=CourseRegistration)
def update_course_registration_history(sender, instance, created, **kwargs):
    if instance.approved:
        d = date.today()
        # if created or changed
        if (instance.course_history.count() == 0 or
                instance.course_history.filter(end=None).exclude(course_id=instance.subject_id).update(end=d)):
            # reopen or create entry starting today
            (
                CourseRegistrationHistory.objects.filter(
                    registration_id=instance.id, course_id=instance.subject_id, start=d,
                ).update(end=None) or
                CourseRegistrationHistory.objects.create(
                    registration_id=instance.id, course_id=instance.subject_id, start=d,
                )
            )



def get_default_agenda():
    return '<p>{}</p>'.format(_('instruction on OSH'))



@python_2_unicode_compatible
class CourseJournalEntry(StartEndMixin, models.Model):
    course      = models.ForeignKey(Course, verbose_name=_('course'), editable=False,
                                    related_name='journal_entries', on_delete=models.PROTECT)
    date        = models.DateField(_('date'))
    start       = models.TimeField(_('start time'), blank=True, null=True,
                                   help_text=_('Leave empty, if the course does not take place'))
    end         = models.TimeField(_('end time'), blank=True, null=True,
                                   help_text=_('Leave empty, if the course does not take place'))
    agenda      = HTMLField(_('session agenda'), default=get_default_agenda)
    registrations = models.ManyToManyField(CourseRegistration, verbose_name=_('participants'), blank=True,
                                           related_name='journal_entries')

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('date', 'start', 'end')
        verbose_name        = _('journal entry')
        verbose_name_plural = _('journal entries')

    def __str__(self):
        return '{course}, {date}'.format(
            course  = self.course.name,
            date    = self.date,
        )

    @cached_property
    def datetime_start(self):
        try:
            return datetime.combine(self.date, self.start)
        except:
            return None

    @cached_property
    def datetime_end(self):
        try:
            return datetime.combine(self.date, self.end)
        except:
            return None

    @cached_property
    def duration(self):
        try:
            return self.datetime_end - self.datetime_start
        except:
            return timedelta()
    duration.short_description = _('duration')

    @cached_property
    def all_registrations(self):
        return list(self.registrations.all())

    @cached_property
    def all_leader_entries(self):
        return list(self.leader_entries.all())

    @cached_property
    def all_leader_entries_by_leader(self):
        return dict((e.timesheet.leader, e) for e in self.all_leader_entries)

    @cached_property
    def all_leaders(self):
        return list(
            le.timesheet.leader for le in self.all_leader_entries
            if le.timesheet.leader in self.course.all_leaders
        )

    @cached_property
    def all_alternates(self):
        return list(
            le.timesheet.leader for le in self.all_leader_entries
            if le.timesheet.leader not in self.course.all_leaders
        )

    @property
    def timesheets(self):
        from .timesheets import Timesheet
        return Timesheet.objects.by_date(self.start).filter(
            leader__in = self.all_leaders + self.all_alternates,
        )

    def save(self, *args, **kwargs):
        if self.end is None:
            self.end = self.start
        super(CourseJournalEntry, self).save(*args, **kwargs)

    def get_edit_url(self):
        return reverse('leprikon:coursejournalentry_update', args=(self.id,))

    def get_delete_url(self):
        return reverse('leprikon:coursejournalentry_delete', args=(self.id,))



@python_2_unicode_compatible
class CourseJournalLeaderEntry(StartEndMixin, models.Model):
    course_entry = models.ForeignKey(CourseJournalEntry, verbose_name=_('course journal entry'),
                                     related_name='leader_entries', editable=False)
    timesheet   = models.ForeignKey('leprikon.Timesheet', verbose_name=_('timesheet'), related_name='course_entries',
                                    editable=False, on_delete=models.PROTECT)
    start       = models.TimeField(_('start time'))
    end         = models.TimeField(_('end time'))

    class Meta:
        app_label           = 'leprikon'
        verbose_name        = _('course journal leader entry')
        verbose_name_plural = _('course journal leader entries')
        unique_together     = (('course_entry', 'timesheet'),)

    def __str__(self):
        return '{}'.format(self.duration)

    @cached_property
    def date(self):
        return self.course_entry.date
    date.short_description = _('date')
    date.admin_order_field = 'course_entry__date'

    @cached_property
    def course(self):
        return self.course_entry.course
    course.short_description = _('course')

    @cached_property
    def datetime_start(self):
        return datetime.combine(self.date, self.start)

    @cached_property
    def datetime_end(self):
        return datetime.combine(self.date, self.end)

    @cached_property
    def duration(self):
        return self.datetime_end - self.datetime_start
    duration.short_description = _('duration')

    @property
    def group(self):
        return self.course

    def get_edit_url(self):
        return reverse('leprikon:coursejournalleaderentry_update', args=(self.id,))

    def get_delete_url(self):
        return reverse('leprikon:coursejournalleaderentry_delete', args=(self.id,))



class CoursePlugin(CMSPlugin):
    course     = models.ForeignKey(Course, verbose_name=_('course'), related_name='+')
    template    = models.CharField(_('template'), max_length=100,
                                   choices=settings.LEPRIKON_COURSE_TEMPLATES,
                                   default=settings.LEPRIKON_COURSE_TEMPLATES[0][0],
                                   help_text=_('The template used to render plugin.'))

    class Meta:
        app_label = 'leprikon'



class CourseListPlugin(CMSPlugin):
    school_year = models.ForeignKey(SchoolYear, verbose_name=_('school year'),
                                    related_name='+', blank=True, null=True)
    course_types = models.ManyToManyField(SubjectType, verbose_name=_('course types'), blank=True, related_name='+',
                                          limit_choices_to={'subject_type': SubjectType.COURSE},
                                          help_text=_('Keep empty to skip searching by course types.'))
    age_groups  = models.ManyToManyField(AgeGroup, verbose_name=_('age groups'), blank=True, related_name='+',
                                         help_text=_('Keep empty to skip searching by age groups.'))
    groups      = models.ManyToManyField(SubjectGroup, verbose_name=_('course groups'), blank=True, related_name='+',
                                         help_text=_('Keep empty to skip searching by groups.'))
    leaders     = models.ManyToManyField(Leader, verbose_name=_('leaders'), blank=True, related_name='+',
                                         help_text=_('Keep empty to skip searching by leaders.'))
    template    = models.CharField(_('template'), max_length=100,
                                   choices=settings.LEPRIKON_COURSELIST_TEMPLATES,
                                   default=settings.LEPRIKON_COURSELIST_TEMPLATES[0][0],
                                   help_text=_('The template used to render plugin.'))

    class Meta:
        app_label = 'leprikon'

    def copy_relations(self, oldinstance):
        self.course_types   = oldinstance.course_types.all()
        self.groups         = oldinstance.groups.all()
        self.age_groups     = oldinstance.age_groups.all()
        self.leaders        = oldinstance.leaders.all()

    @cached_property
    def all_course_types(self):
        return list(self.course_types.all())

    @cached_property
    def all_age_groups(self):
        return list(self.age_groups.all())

    @cached_property
    def all_groups(self):
        return list(self.groups.all())

    @cached_property
    def all_leaders(self):
        return list(self.leaders.all())

    Group = namedtuple('Group', ('group', 'objects'))

    def render(self, context):
        school_year = (self.school_year or getattr(context.get('request'), 'school_year') or
                       SchoolYear.objects.get_current())
        courses = Course.objects.filter(school_year=school_year, public=True).distinct()

        if self.all_course_types:
            courses = courses.filter(subject_type__in = self.all_course_types)
        if self.all_age_groups:
            courses = courses.filter(age_groups__in = self.all_age_groups)
        if self.all_leaders:
            courses = courses.filter(leaders__in = self.all_leaders)
        if self.all_groups:
            courses = courses.filter(groups__in = self.all_groups)
            groups = self.all_groups
        elif self.all_course_types:
            groups = SubjectGroup.objects.filter(subject_types__in = self.all_course_types)
        else:
            groups = SubjectGroup.objects.all()

        context.update({
            'school_year':  school_year,
            'courses':      courses,
            'groups':       (
                self.Group(group = group, objects = courses.filter(groups=group))
                for group in groups
            ),
        })
        return context



class FilteredCourseListPlugin(CMSPlugin):
    school_year = models.ForeignKey(SchoolYear, verbose_name=_('school year'),
                                    related_name='+', blank=True, null=True)
    course_types = models.ManyToManyField(SubjectType, verbose_name=_('course types'), related_name='+',
                                          limit_choices_to={'subject_type': SubjectType.COURSE})

    class Meta:
        app_label = 'leprikon'

    def copy_relations(self, oldinstance):
        self.course_types = oldinstance.course_types.all()

    @cached_property
    def all_course_types(self):
        return list(self.course_types.all())

    def render(self, context):
        school_year = (self.school_year or getattr(context.get('request'), 'school_year') or
                       SchoolYear.objects.get_current())

        from ..forms.subjects import SubjectFilterForm
        form = SubjectFilterForm(
            subject_type_type = SubjectType.COURSE,
            subject_types = self.all_course_types,
            school_year = school_year,
            is_staff = context['request'].user.is_staff,
            data=context['request'].GET,
        )
        context.update({
            'school_year':  school_year,
            'form':         form,
            'courses':      form.get_queryset(),
        })
        return context



from __future__ import unicode_literals

from collections import namedtuple

from django.core.urlresolvers import reverse_lazy as reverse
from django.db.models import Sum
from django.template.response import TemplateResponse
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from . import ReportBaseView
from ...forms.reports.events import EventPaymentsForm, EventPaymentsStatusForm
from ...models.events import Event
from ...models.subjects import SubjectPayment, SubjectType
from ...models.utils import PaymentStatus


class ReportEventPaymentsView(ReportBaseView):
    form_class      = EventPaymentsForm
    template_name   = 'leprikon/reports/event_payments.html'
    title           = _('Event payments')
    submit_label    = _('Show')
    back_url        = reverse('leprikon:report_list')

    def form_valid(self, form):
        context = form.cleaned_data
        context['form'] = form
        context['payments'] = SubjectPayment.objects.filter(
            registration__subject__subject_type__subject_type=SubjectType.EVENT,
            created__gte=context['date_start'],
            created__lte=context['date_end'],
        )
        context['sum'] = context['payments'].aggregate(Sum('amount'))['amount__sum']
        return TemplateResponse(self.request, self.template_name, self.get_context_data(**context))



class ReportEventPaymentsStatusView(ReportBaseView):
    form_class      = EventPaymentsStatusForm
    template_name   = 'leprikon/reports/event_payments_status.html'
    title           = _('Event payments status')
    submit_label    = _('Show')
    back_url        = reverse('leprikon:report_list')

    EventPaymentsStatusSums = namedtuple('EventPaymentsStatusSums', ('registrations', 'status'))

    def form_valid(self, form):
        context = form.cleaned_data
        context['form'] = form
        context['reports'] = [
            self.Report(event, context['date'])
            for event in Event.objects.filter(school_year=self.request.school_year)
        ]
        context['sum'] = self.EventPaymentsStatusSums(
            registrations   = sum(len(r.registrations)  for r in context['reports']),
            status          = sum(r.status              for r in context['reports']),
        )
        return TemplateResponse(self.request, self.template_name, self.get_context_data(**context))

    class Report:
        def __init__(self, event, d):
            self.event = event
            self.date = d

        @cached_property
        def registrations(self):
            return list(self.event.registrations.filter(
                created__lte=self.date,
            ))

        RegPaymentStatus = namedtuple('RegPaymentStatus', ('registration', 'status'))

        @cached_property
        def registration_statuses(self):
            return [
                self.RegPaymentStatus(
                    registration = registration,
                    status       = registration.eventregistration.get_payment_status(self.date),
                )
                for registration in self.registrations
            ]

        @cached_property
        def status(self):
            return PaymentStatus(
                price       = sum(rs.status.price    for rs in self.registration_statuses),
                paid        = sum(rs.status.paid     for rs in self.registration_statuses),
                discount    = sum(rs.status.discount for rs in self.registration_statuses),
            )



from __future__ import unicode_literals

from django.apps import AppConfig
from django.db.models.signals import post_migrate, post_save
from django.utils.translation import ugettext_lazy as _


def create_leprikonsites(sender, **kwargs):
    from .models.leprikonsite import LeprikonSite, Site

    for site in Site.objects.filter(leprikonsite=None):
        LeprikonSite.objects.create(site_ptr=site, **{f.name: getattr(site, f.name) for f in Site._meta.fields})


class LeprikonConfig(AppConfig):
    name            = 'leprikon'
    verbose_name    = _('Leprikon')

    def ready(self):
        from django.contrib.sites.models import Site
        post_migrate.connect(create_leprikonsites, sender=self)
        post_save.connect(create_leprikonsites, sender=Site)



from __future__ import unicode_literals

from . import (
    agegroup, courses, events, fields, insurance, leprikonsite, messages,
    place, printsetup, question, roles, school, schoolyear, startend, subjects,
    timesheets, utils,
)

(
    agegroup,
    courses,
    events,
    fields,
    insurance,
    leprikonsite,
    messages,
    place,
    printsetup,
    question,
    roles,
    school,
    schoolyear,
    startend,
    subjects,
    timesheets,
    utils,
)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-18 17:03
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import djangocms_text_ckeditor.fields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('leprikon', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sent', models.DateTimeField(auto_now_add=True, verbose_name='sent')),
                ('subject', models.CharField(max_length=150, verbose_name='subject')),
                ('text', djangocms_text_ckeditor.fields.HTMLField(default='', verbose_name='text')),
            ],
            options={
                'ordering': ('-sent',),
                'verbose_name': 'message',
                'verbose_name_plural': 'messages',
            },
        ),
        migrations.CreateModel(
            name='MessageRecipient',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(editable=False)),
                ('sent', models.DateTimeField(auto_now_add=True, verbose_name='sent')),
                ('received', models.DateTimeField(default=None, null=True, verbose_name='received')),
                ('message', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='recipients', to='leprikon.Message', verbose_name='message')),
                ('recipient', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='leprikon_messages', to=settings.AUTH_USER_MODEL, verbose_name='recipient')),
            ],
            options={
                'ordering': ('-message__sent',),
            },
        ),
        migrations.AlterUniqueTogether(
            name='messagerecipient',
            unique_together=set([('message', 'recipient')]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2017-01-12 21:23
from __future__ import unicode_literals

from django.db import migrations
import leprikon.models.fields


def populate_registration_prices(apps, schema_editor):
    ClubRegistration = apps.get_model('leprikon', 'clubregistration')
    EventRegistration = apps.get_model('leprikon', 'eventregistration')

    for reg in ClubRegistration.objects.all():
        reg.price = reg.club.price
        reg.save()

    for reg in EventRegistration.objects.all():
        reg.price = reg.event.price
        reg.save()


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0022_club_registration_history'),
    ]

    operations = [
        migrations.AddField(
            model_name='clubregistration',
            name='price',
            field=leprikon.models.fields.PriceField(decimal_places=0, default=0, editable=False, max_digits=10, verbose_name='price'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='eventregistration',
            name='price',
            field=leprikon.models.fields.PriceField(decimal_places=0, default=0, editable=False, max_digits=10, verbose_name='price'),
            preserve_default=False,
        ),
        migrations.RunPython(populate_registration_prices),
    ]



from __future__ import unicode_literals

from django.contrib import admin

from ..models.schoolyear import SchoolYearPeriod


class SchoolYearPeriodInlineAdmin(admin.TabularInline):
    model = SchoolYearPeriod
    extra = 0


class SchoolYearAdmin(admin.ModelAdmin):
    list_display    = ('name', 'active')
    list_editable   = ('active',)
    inlines         = (SchoolYearPeriodInlineAdmin,)

    # do not allow to delete entries in admin
    def has_delete_permission(self, request, obj=None):
        return False

    def get_readonly_fields(self, request, obj=None):
        return obj and ('year',) or ()

    def get_actions(self, request):
        actions = super(SchoolYearAdmin, self).get_actions(request)
        if 'delete_selected' in actions:
            del(actions['delete_selected'])
        return actions



from __future__ import unicode_literals

from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _


class StartEndMixin(object):
    def validate(self):
        if self.start and self.end and self.start > self.end:
            raise ValidationError({
                'start': [_('Start must be before end')],
                'end': [_('End must be later than start')],
            })

    def clean_fields(self, exclude=None):
        super(StartEndMixin, self).clean_fields(exclude)
        self.validate()

    def save(self, *args, **kwargs):
        self.validate()
        super(StartEndMixin, self).save(*args, **kwargs)



from __future__ import unicode_literals

from django import forms
from django.contrib import admin
from django.contrib.admin.templatetags.admin_list import _boolean_icon
from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _

from ..models.events import Event, EventRegistration
from ..models.schoolyear import SchoolYear
from ..models.subjects import SubjectType
from ..utils import currency
from .filters import (
    ApprovedListFilter, CanceledListFilter, EventGroupListFilter,
    EventListFilter, EventTypeListFilter, LeaderListFilter,
    SchoolYearListFilter,
)
from .subjects import (
    SubjectBaseAdmin, SubjectPaymentAdmin, SubjectRegistrationBaseAdmin,
)


class EventAdmin(SubjectBaseAdmin):
    subject_type = SubjectType.EVENT
    list_display    = (
        'id', 'name', 'subject_type', 'get_groups_list', 'get_leaders_list',
        'get_times_list',
        'place', 'public', 'registration_allowed_icon',
        'get_registrations_link',
        'icon', 'note',
    )
    list_export     = (
        'id', 'name', 'subject_type', 'get_groups_list', 'get_leaders_list',
        'start_date', 'start_time', 'end_date', 'end_time',
        'place', 'public',
        'get_registrations_count', 'note',
    )
    list_filter     = (
        ('school_year',     SchoolYearListFilter),
        ('subject_type',    EventTypeListFilter),
        ('groups',          EventGroupListFilter),
        ('leaders',         LeaderListFilter),
    )
    date_hierarchy  = 'start_date'
    actions         = (
        'publish', 'unpublish',
        'copy_to_school_year',
    )

    def publish(self, request, queryset):
        Event.objects.filter(id__in=[reg['id'] for reg in queryset.values('id')]).update(public = True)
        self.message_user(request, _('Selected events were published.'))
    publish.short_description = _('Publish selected events')

    def unpublish(self, request, queryset):
        Event.objects.filter(id__in=[reg['id'] for reg in queryset.values('id')]).update(public = False)
        self.message_user(request, _('Selected events were unpublished.'))
    unpublish.short_description = _('Unpublish selected events')

    def copy_to_school_year(self, request, queryset):
        class SchoolYearForm(forms.Form):
            school_year = forms.ModelChoiceField(
                label=_('Target school year'),
                help_text=_('All selected events will be copied to selected school year.'),
                queryset=SchoolYear.objects.all(),
            )
        if request.POST.get('post', 'no') == 'yes':
            form = SchoolYearForm(request.POST)
            if form.is_valid():
                school_year = form.cleaned_data['school_year']
                for event in queryset.all():
                    event.copy_to_school_year(school_year)
                self.message_user(request, _('Selected events were copied to school year {}.').format(school_year))
                return
        else:
            form = SchoolYearForm()
        return render_to_response(
            'leprikon/admin/action_form.html', {
                'title': _('Select target school year'),
                'queryset': queryset,
                'opts': self.model._meta,
                'form': form,
                'action': 'copy_to_school_year',
                'action_checkbox_name': admin.helpers.ACTION_CHECKBOX_NAME,
            }, context_instance=RequestContext(request))
    copy_to_school_year.short_description = _('Copy selected events to another school year')

    def get_message_recipients(self, request, queryset):
        return get_user_model().objects.filter(
            leprikon_registrations__subject__in = queryset
        ).distinct()

    def get_registrations_link(self, obj):
        icon = False
        if obj.registrations_count == 0:
            title = _('There are no registrations for this event.')
        elif obj.min_count is not None and obj.registrations_count < obj.min_count:
            title = _('The number of registrations is lower than {}.').format(obj.min_count)
        elif obj.max_count is not None and obj.registrations_count > obj.max_count:
            title = _('The number of registrations is greater than {}.').format(obj.max_count)
        else:
            icon = True
            title = ''
        return '<a href="{url}" title="{title}">{icon} {count}</a>'.format(
            url     = reverse('admin:{}_{}_changelist'.format(
                EventRegistration._meta.app_label,
                EventRegistration._meta.model_name,
            )) + '?subject={}'.format(obj.id),
            title   = title,
            icon    = _boolean_icon(icon),
            count   = obj.registrations_count,
        )
    get_registrations_link.short_description = _('registrations')
    get_registrations_link.admin_order_field = 'registrations_count'
    get_registrations_link.allow_tags = True



class EventRegistrationAdmin(SubjectRegistrationBaseAdmin):
    list_display    = (
        'id', 'download_tag', 'subject_name', 'participant', 'price', 'event_discounts', 'event_payments',
        'created', 'approved', 'cancel_request', 'canceled',
    )
    list_filter     = (
        ('subject__school_year',    SchoolYearListFilter),
        ('subject__subject_type',   EventTypeListFilter),
        ApprovedListFilter,
        CanceledListFilter,
        ('subject',                 EventListFilter),
        ('subject__leaders',        LeaderListFilter),
    )

    def event_discounts(self, obj):
        status = obj.get_payment_status()
        return format_html(
            '<a target="_blank" href="{href_list}"><b>{amount}</b></a> &nbsp; '
            '<a target="_blank" class="addlink" href="{href_add}" style="background-position: 0 0" title="{add}"></a>',
            href_list   = reverse('admin:leprikon_eventdiscount_changelist') + '?registration={}'.format(obj.id),
            href_add    = reverse('admin:leprikon_eventdiscount_add') + '?registration={}'.format(obj.id),
            add         = _('add discount'),
            amount      = currency(status.discount),
        )
    event_discounts.allow_tags = True
    event_discounts.short_description = _('event discounts')

    def event_payments(self, obj):
        status = obj.get_payment_status()
        return format_html(
            '<a target="_blank" style="color: {color}" href="{href_list}" title="{title}"><b>{amount}</b></a> &nbsp; '
            '<a target="_blank" class="addlink" href="{href_add}" style="background-position: 0 0" title="{add}"></a>',
            color       = status.color,
            href_list   = reverse('admin:leprikon_subjectpayment_changelist') + '?registration={}'.format(obj.id),
            href_add    = reverse('admin:leprikon_subjectpayment_add') + '?registration={}'.format(obj.id),
            title       = status.title,
            add         = _('add payment'),
            amount      = currency(status.paid),
        )
    event_payments.allow_tags = True
    event_payments.short_description = _('event payments')



class EventDiscountAdmin(SubjectPaymentAdmin):
    list_display    = ('created', 'registration', 'subject', 'amount_html', 'explanation')

    def get_model_perms(self, request):
        return {}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from setuptools import setup, find_packages

from leprikon import __version__

setup(
    name            = 'leprikon',
    version         = __version__,
    description     = 'Django CMS based IS for leisure centre',
    author          = 'Jakub Dorňák',
    author_email    = 'jakub.dornak@misli.cz',
    license         = 'BSD',
    url             = 'https://github.com/leprikon-cz/leprikon',
    packages        = find_packages(),
    include_package_data = True,
    install_requires=[
        'PyPDF2',
        'cmsplugin-filer',
        'django>=1.9',
        'django-cms>=3.4',
        'django-countries',
        'django-filer',
        'django-ganalytics',
        'django-haystack',
        'django-localflavor',
        'djangocms_text_ckeditor',
        'trml2pdf',
    ],
    classifiers     = [
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Natural Language :: Czech',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)



from __future__ import unicode_literals

from django import forms
from django.utils.translation import ugettext_lazy as _

from ..form import FormMixin


class EventPaymentsForm(FormMixin, forms.Form):
    date_start  = forms.DateField(label=_('Start date'))
    date_end    = forms.DateField(label=_('End date'))



class EventPaymentsStatusForm(FormMixin, forms.Form):
    date    = forms.DateField(label=_('Status for date'))



from __future__ import unicode_literals

from django import forms
from django.utils.translation import ugettext_lazy as _

from ..conf import settings


class SupportForm(forms.Form):
    question = forms.CharField(
        label=_('Help needed?'),
        widget=forms.Textarea(attrs = {'class': 'form-control'}),
        required=False,
    )
    back = settings.LEPRIKON_PARAM_BACK



import os
from django.utils.translation import ugettext_lazy as _

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

SECRET_KEY = 'l!92en*ntoe7t&6$l_j@pqhtv5#h!0yrn%ot#*w^)u0-ww@*vn'

DEBUG = True

ALLOWED_HOSTS = ['*']


# Application definition

INSTALLED_APPS = [
    'leprikon',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',
    'djangocms_text_ckeditor',
    'treebeard',
    'cms',
    'menus',
    'mptt',
    'easy_thumbnails',
    'sekizai',
    'filer',
    'cmsplugin_filer_file',
    'cmsplugin_filer_folder',
    'cmsplugin_filer_image',
    'cmsplugin_filer_link',
    'cmsplugin_filer_teaser',
    'cmsplugin_filer_utils',
    'cmsplugin_filer_video',
    'ganalytics',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'leprikon_tests.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'leprikon_tests.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.10/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}


# Password validation
# https://docs.djangoproject.com/en/1.10/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/1.10/topics/i18n/

LANGUAGE_CODE = 'cs'

LANGUAGES = [
    ('cs', _('Czech')),
]

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.10/howto/static-files/

STATIC_URL = '/static/'

SITE_ID = 1

CMS_LANGUAGES = {
    SITE_ID: [{'code': l[0], 'name': l[1]} for l in LANGUAGES],
    'default': {
        'public': True,
        'fallbacks': [LANGUAGE_CODE],
        'hide_untranslated': True,
        'redirect_on_fallback': True,
    },
}

HAYSTACK_CONNECTIONS = {
    'default': {
        'ENGINE': 'haystack.backends.whoosh_backend.WhooshEngine',
        'PATH': os.path.join(BASE_DIR, 'data', 'search'),
    },
    'cs': {
        'ENGINE': 'haystack.backends.whoosh_backend.WhooshEngine',
        'PATH': os.path.join(BASE_DIR, 'data', 'search'),
    },
}




# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf import settings as wrapped_settings

from . import default_settings as default_settings


class LazySettings(object):

    def __dir__(self):
        return dir(wrapped_settings) + dir(default_settings)

    def __getattr__(self, name):
        try:
            value = getattr(wrapped_settings, name)
        except AttributeError:
            value = getattr(default_settings, name)
        setattr(self, name, value)
        return value


settings = LazySettings()



from __future__ import unicode_literals

from django import forms

from ..models.roles import Participant
from .form import FormMixin


class ParticipantForm(FormMixin, forms.ModelForm):

    def __init__(self, user, **kwargs):
        super(ParticipantForm, self).__init__(**kwargs)
        self.instance.user = user

    class Meta:
        model = Participant
        fields = [
            'first_name', 'last_name', 'birth_num', 'age_group',
            'street', 'city', 'postal_code', 'citizenship', 'insurance',
            'email', 'phone',
            'school', 'school_other', 'school_class',
            'health',
        ]
